@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014, 2015 Red Hat
+ * Copyright (C) 2014-2017 Red Hat
  *
  * Author: Nikos Mavrogiannopoulos
  *
@@ -45,6 +45,7 @@
 #include <sys/un.h>
 #include <assert.h>
 #include "hex.h"
+#include "geoip.h"
 #include <vpn.h>
 #include <base64-helper.h>
 
@@ -207,8 +208,10 @@ int handle_status_cmd(struct unix_ctx *ctx, const char *arg, cmd_params_st *para
 	init_reply(&raw);
 
 	print_start_block(stdout, params);
-	if (NO_JSON(params))
-		printf("OpenConnect SSL VPN server\n");
+	if (NO_JSON(params)) {
+		printf("Note: the printed statistics are not real-time; session time\n");
+		printf("as well as RX and TX data are updated on user disconnect\n");
+	}
 
 	ret = send_cmd(ctx, CTL_CMD_STATUS, NULL, NULL, NULL, &raw);
 	if (ret < 0) {
@@ -220,21 +223,73 @@ int handle_status_cmd(struct unix_ctx *ctx, const char *arg, cmd_params_st *para
 		goto error_status;
 
 
-	print_single_value(stdout, params, "Status", rep->status != 0 ? "online" : "error", 1);
+	if (rep->status) {
+		print_separator(stdout, params);
 
-	t = rep->start_time;
-	tm = localtime(&t);
-	print_time_ival7(buf, time(0), t);
-	strftime(str_since, sizeof(str_since), DATE_TIME_FMT, tm);
+		if (NO_JSON(params))
+			printf("General info:\n");
+
+		print_single_value(stdout, params, "Status", rep->status != 0 ? "online" : "error", 1);
+		print_single_value_int(stdout, params, "Server PID", rep->pid, 1);
+		print_single_value_int(stdout, params, "Sec-mod PID", rep->sec_mod_pid, 0);
+
+		t = rep->start_time;
+		tm = localtime(&t);
+		print_time_ival7(buf, time(0), t);
+		strftime(str_since, sizeof(str_since), DATE_TIME_FMT, tm);
+
+		print_single_value_ex(stdout, params, "Up since", str_since, buf, 1);
+		print_single_value_int(stdout, params, "Active sessions", rep->active_clients, 1);
+		print_single_value_int(stdout, params, "Total sessions", rep->total_sessions_closed, 1);
+		print_single_value_int(stdout, params, "Total authentication failures", rep->total_auth_failures, 1);
+		print_single_value_int(stdout, params, "IPs in ban list", rep->banned_ips, 1);
+		if (params->debug) {
+			print_single_value_int(stdout, params, "Sec-mod client entries", rep->secmod_client_entries, 1);
+			print_single_value_int(stdout, params, "TLS DB entries", rep->stored_tls_sessions, 1);
+		}
+
+		print_separator(stdout, params);
+		if (NO_JSON(params))
+			printf("Current stats period:\n");
+
+		t = rep->last_reset;
+		if (t > 0) {
+			tm = localtime(&t);
+			print_time_ival7(buf, time(0), t);
+			strftime(str_since, sizeof(str_since), DATE_TIME_FMT, tm);
+
+			print_single_value_ex(stdout, params, "Last stats reset", str_since, buf, 1);
+		}
+
+		print_single_value_int(stdout, params, "Sessions handled", rep->sessions_closed, 1);
+		print_single_value_int(stdout, params, "Timed out sessions", rep->session_timeouts, 1);
+		print_single_value_int(stdout, params, "Timed out (idle) sessions", rep->session_idle_timeouts, 1);
+		print_single_value_int(stdout, params, "Closed due to error sessions", rep->session_errors, 1);
+		print_single_value_int(stdout, params, "Authentication failures", rep->auth_failures, 1);
+
+		print_time_ival7(buf, rep->avg_auth_time, 0);
+		print_single_value(stdout, params, "Average auth time", buf, 1);
+
+		print_time_ival7(buf, rep->max_auth_time, 0);
+		print_single_value(stdout, params, "Max auth time", buf, 1);
+
+		print_time_ival7(buf, rep->avg_session_mins*60, 0);
+		print_single_value(stdout, params, "Average session time", buf, 1);
+
+		print_time_ival7(buf, rep->max_session_mins*60, 0);
+		print_single_value(stdout, params, "Max session time", buf, 1);
+
+		bytes2human(rep->kbytes_in*1000, buf, sizeof(buf), "");
+		print_single_value(stdout, params, "RX", buf, 1);
+		bytes2human(rep->kbytes_out*1000, buf, sizeof(buf), "");
+		print_single_value(stdout, params, "TX", buf, 1);
+
+		if (rep->min_mtu > 0)
+			print_single_value_int(stdout, params, "Min MTU", rep->min_mtu, 1);
+		if (rep->max_mtu > 0)
+			print_single_value_int(stdout, params, "Max MTU", rep->max_mtu, 1);
+	}
 
-	print_single_value_ex(stdout, params, "Up since", str_since, buf, 1);
-	print_single_value_int(stdout, params, "Clients", rep->active_clients, 1);
-	print_single_value_int(stdout, params, "Sec-mod client entries", rep->secmod_client_entries, 1);
-	print_single_value_int(stdout, params, "IPs in ban list", rep->banned_ips, 1);
-	print_single_value_int(stdout, params, "TLS DB entries", rep->stored_tls_sessions, 1);
-	print_separator(stdout, params);
-	print_single_value_int(stdout, params, "Server PID", rep->pid, 1);
-	print_single_value_int(stdout, params, "Sec-mod PID", rep->sec_mod_pid, 0);
 	print_end_block(stdout, params, 0);
 
 	status_rep__free_unpacked(rep, &pa);
@@ -995,7 +1050,8 @@ int common_info_cmd(UserListRep * args, FILE *out, cmd_params_st *params)
 			print_pair_value(out, params, "Device", args->user[i]->tun, "MTU", int2str(tmpbuf, args->user[i]->mtu), 1);
 		else
 			print_single_value(out, params, "Device", args->user[i]->tun, 1);
-		print_pair_value(out, params, "Remote IP", args->user[i]->ip, "Local Device IP", args->user[i]->local_dev_ip, 1);
+		print_pair_value(out, params, "Remote IP", args->user[i]->ip, "Location", geo_lookup(args->user[i]->ip, tmpbuf, sizeof(tmpbuf)), 1);
+		print_single_value(out, params, "Local Device IP", args->user[i]->local_dev_ip, 1);
 
 		if (args->user[i]->local_ip != NULL && args->user[i]->local_ip[0] != 0 &&
 		    args->user[i]->remote_ip != NULL && args->user[i]->remote_ip[0] != 0) {
